﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CMS.BusinessEntities;

namespace CMS.DataAccess
{
    public class n2FileManager : ManagerBase 
    {
        private readonly N2ModelDataContext db;

        public n2FileManager()
        {
            db = new N2ModelDataContext(ConnectionString);
        }

        public n2FileManager(N2ModelDataContext context)
        {
            db = context;
        }

        public n2File GetRootFolder()
        {
            n2File root = null;
            var query = from f in db.n2Files
                        where f.ParentId == null
                        select f;
            if (query.Count() > 0)
                root = query.Single<n2File>();
            return root;
        }

        public n2File GetFileById(int id)
        {
            n2File file = null;
            var query = from f in db.n2Files
                        where f.ID == id
                        select f;
            if (query.Count() > 0)
                file = query.Single<n2File>();
            return file;
        }

        public n2File GetFileByFullPath(string fullPath)
        {
            n2File file = null;
            var query = from f in db.n2Files
                        where f.FullPath == fullPath
                        select f;
            if (query.Count() > 0)
                file = query.Single<n2File>();
            return file;
        }

        public n2File AddFile(string path, string name, byte[] content, string contentType)
        {
            return AddFile(GetFileByFullPath(path), name, content, contentType);
        }


        private bool ChildExists(n2File parent, string name)
        {
            var query = from f in db.n2Files
                        where f.ParentId == parent.ID
                        && f.Name == name
                        select f;
            return (query.Count() > 0);
        }

        private void ValidateAdd(n2File parent, string name)
        {
            if (parent == null)
                throw new ArgumentException(string.Format("Parent cannot be when adding a child (file or directory)"));
            if (!parent.IsDirectory)
                throw new ArgumentException(string.Format("A child can only be added to a Directory, id={0} ({1}) is not a Directory.", parent.ParentId, parent.FullPath), "ParentId");
            if (ChildExists(parent, name))
                throw new InvalidOperationException(string.Format("A child named {0} already exists in {1}", name, parent.FullPath));
        }

        public n2File AddFile(n2File parent, string name, byte[] content, string contentType)
        {
            ValidateAdd(parent, name);
            n2File file = new n2File()
            {
                IsDirectory = false,
                ParentId = parent.ID,
                Name = name,
                FullPath = ((parent.FullPath == "/") ? "" : parent.FullPath) + "/" + name,
                FileContent = content,
                ContentType = contentType
            };
            db.n2Files.InsertOnSubmit(file);
            db.SubmitChanges();
            return file;
        }

        public n2File AddDirectory(string path, string name)
        {
            return AddDirectory(GetFileByFullPath(path), name);
        }

        public n2File AddDirectory(n2File parent, string name)
        {
            ValidateAdd(parent, name);
            n2File directory = new n2File()
            {
                IsDirectory = true,
                ParentId = parent.ID,
                Name = name,
                FullPath = ((parent.FullPath == "/") ? "" : parent.FullPath) + "/" + name,
                FileContent = null,
                ContentType = null
            };
            db.n2Files.InsertOnSubmit(directory);
            db.SubmitChanges();
            return directory;
        }

        public List<n2File> GetChildren(n2File parent, bool isDirectory)
        {
            if (parent == null)
                throw new ArgumentException("Parent cannot be null", "parent");
            if (!parent.IsDirectory)
                throw new InvalidOperationException("Only directories have children");
            var files = from f in db.n2Files
                        where f.ParentId == parent.ID
                        && f.IsDirectory == isDirectory 
                        select f;
            return files.ToList<n2File>();
        }

        public List<n2File> GetChildrenByFullPath(string fullpath, bool isDirectory)
        {
            return GetChildren(GetFileByFullPath(fullpath), isDirectory);
        }

        public List<n2File> GetChildrenById(int id, bool isDirectory)
        {
            return GetChildren(GetFileById(id), isDirectory);
        }


        public void Delete(string path)
        {
            n2File file = GetFileByFullPath(path);
            if (file != null)
            {
                if ((!file.IsDirectory) && FileIsReferenced(file))
                    throw new InvalidOperationException("You cannot delete a file that is referenced by a content item.");
                if(HasChildren(file))
                    throw new InvalidOperationException("You cannot delete a directory that has child files/directories.");
                db.n2Files.DeleteOnSubmit(file);
                db.SubmitChanges();
            }
        }

        private IEnumerable<n2Detail> FileReferences(n2File file)
        {
            var query = from d in db.n2Details
                        where d.Name == "Asset"
                        && d.StringValue == file.FullPath 
                        select d;
            return query.AsEnumerable<n2Detail>();
        }

        private bool FileIsReferenced(n2File file)
        {
            IEnumerable<n2Detail> query = FileReferences(file);
            return (query.Count() > 0);
        }

        private bool HasChildren(n2File parent)
        {
            if (!parent.IsDirectory)
                return false;
            var query = from f in db.n2Files
                        where f.ParentId == parent.ID
                        select f;
            return (query.Count() > 0);
        }

    }
}
