﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Web;
using System.Collections.Specialized;

namespace CMS.Web.HttpHandlers
{
    public abstract class HandlerBase : IHttpHandler
    {
        private HttpRequest _Request;
        private HttpResponse _Response;
        private HttpServerUtility _Server;
        private NameValueCollection _QueryString;
        private NameValueCollection _Form;
        protected event EventHandler ProcessingRequest;

        protected HandlerBase()
        {
            ProcessingRequest += new EventHandler(OnProcessingRequest);
        }

        protected abstract void OnProcessingRequest(object sender, EventArgs e);

        public void ProcessRequest(HttpContext context)
        {
            Request = context.Request;
            Response = context.Response;
            Server = context.Server;
            QueryString = context.Request.QueryString;
            Form = context.Request.Form;

            ProcessRequest();
        }

        public virtual void ProcessRequest()
        {
            if (ProcessingRequest != null)
                ProcessingRequest(this, new EventArgs());
        }

        public NameValueCollection QueryString
        {
            get
            {
                if (_QueryString == null && Request == null)
                {
                    _QueryString = new NameValueCollection();
                }
                return _QueryString;
            }
            set { _QueryString = value; }
        }

        public NameValueCollection Form
        {
            get
            {
                if (_Form == null && Request == null)
                {
                    _Form = new NameValueCollection();
                }
                return _Form;
            }
            set { _Form = value; }
        }

        public HttpRequest Request
        {
            get { return _Request; }
            set { _Request = value; }
        }

        public HttpResponse Response
        {
            get { return _Response; }
            set { _Response = value; }
        }

        public HttpServerUtility Server
        {
            get { return _Server; }
            set { _Server = value; }
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }

        protected NameValueCollection Q
        {
            get
            {
                return QueryString;
            }
        }


        #region long value

        protected int? QInt(string key)
        {
            return NameValueInt(key, QueryString);
        }

        protected int? FInt(string key)
        {
            return NameValueInt(key, Form);
        }

        protected int? GetInt(string key)
        {
            int? maybeInt = FInt(key);
            if (maybeInt.HasValue)
                return maybeInt;
            else
                return QInt(key);
        }

        private int? NameValueInt(string key, NameValueCollection col)
        {
            string maybeInt = col[key];
            if (string.IsNullOrEmpty(maybeInt))
            {
                return null;
            }
            int result;
            if (int.TryParse(maybeInt, out result))
            {
                return result;
            }
            else
            {
                return null;
            }
        }

        #endregion

        #region long value

        protected long? QLong(string key)
        {
            return NameValueLong(key, QueryString);
        }

        protected long? FLong(string key)
        {
            return NameValueLong(key, Form);
        }

        protected long? GetLong(string key)
        {
            long? maybeLong = FLong(key);
            if (maybeLong.HasValue)
                return maybeLong;
            else
                return QLong(key);
        }

        private long? NameValueLong(string key, NameValueCollection col)
        {
            string maybeLong = col[key];
            if (string.IsNullOrEmpty(maybeLong))
            {
                return null;
            }
            long result;
            if (long.TryParse(maybeLong, out result))
            {
                return result;
            }
            else
            {
                return null;
            }
        }

        #endregion

        #region bool value

        protected bool? GetBool(string key)
        {
            bool? maybeBool = FBool(key);
            if (maybeBool.HasValue)
                return maybeBool;
            else
                return QBool(key);
        }

        protected bool? FBool(string key)
        {
            return NameValueBool(key, Form);
        }

        protected bool? QBool(string key)
        {
            return NameValueBool(key, QueryString);
        }

        private bool? NameValueBool(string key, NameValueCollection col)
        {
            string maybeBool = col[key];
            if (string.IsNullOrEmpty(maybeBool))
            {
                return null;
            }
            bool result;
            if (bool.TryParse(maybeBool, out result))
            {
                return result;
            }
            else
            {
                return null;
            }
        }

        #endregion

        #region string value

        protected string GetString(string key)
        {
            string maybeString = FString(key);
            if (string.IsNullOrEmpty(maybeString))
                maybeString = QString(key);
            return maybeString;
        }

        protected string QString(string key, bool nullIfEmpty)
        {
            return NameValueString(key, QueryString, false);
        }

        protected string QString(string key)
        {
            return NameValueString(key, QueryString, true);
        }

        protected string FString(string key, bool nullIfEmpty)
        {
            return NameValueString(key, Form, false);
        }

        protected string FString(string key)
        {
            return NameValueString(key, Form, true);
        }

        private string NameValueString(string key, NameValueCollection col, bool nullIfEmpty)
        {
            string maybeString = col[key];
            if (string.IsNullOrEmpty(maybeString))
            {
                if (nullIfEmpty)
                {
                    return null;
                }
                else
                {
                    return string.Empty;
                }
            }
            else
                return maybeString;
        }

        #endregion

    }

}
